from dateutil.easter import EASTER_WESTERN

from holidata.holiday import Country
from holidata.utils import day, first

"""
source: https://almanakka.helsinki.fi/en/flag-days-and-holidays-in-finland.html
"""


class FI(Country):
    id = "FI"
    languages = ["fi", "sv"]
    easter_type = EASTER_WESTERN

    def __init__(self):
        super().__init__()

        self.define_holiday() \
            .with_names({
                "fi": "Uudenvuodenpäivä",
                "sv": "Nyårsdagen",
            }) \
            .on(month=1, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_names({
                "fi": "Loppiainen",
                "sv": "Trettondedagen",
            }) \
            .on(month=1, day=6) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_names({
                "fi": "Vappu",
                "sv": "Första maj",
            }) \
            .on(month=5, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_names({
                "fi": "Itsenäisyyspäivä",
                "sv": "Självständighetsdagen",
            }) \
            .on(month=12, day=6) \
            .with_flags("NF")

        self.define_holiday() \
            .with_names({
                "fi": "Joulupäivä",
                "sv": "Juldagen",
            }) \
            .on(month=12, day=25) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_names({
                "fi": "Tapaninpäivä",
                "sv": "Annandag jul",
            }) \
            .on(month=12, day=26) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_names({
                "fi": "Pitkäperjantai",
                "sv": "Långfredagen",
            }) \
            .on(day(2).before(self.easter())) \
            .with_flags("NRV")

        self.define_holiday() \
            .with_names({
                "fi": "Pääsiäispäivä",
                "sv": "Påskdagen",
            }) \
            .on(self.easter()) \
            .with_flags("NRV")

        self.define_holiday() \
            .with_names({
                "fi": "2. pääsiäispäivä",
                "sv": "Annandag påsk",
            }) \
            .on(day(1).after(self.easter())) \
            .with_flags("NRV")

        self.define_holiday() \
            .with_names({
                "fi": "Helatorstai",
                "sv": "Kristi himmelfärdsdag",
            }) \
            .on(day(39).after(self.easter())) \
            .with_flags("NRV")

        self.define_holiday() \
            .with_names({
                "fi": "Helluntaipäivä",
                "sv": "Pingst",
            }) \
            .on(day(49).after(self.easter())) \
            .with_flags("NRV")

        """
        Saturday between 20 and 26 June: Midsummer Day
        """
        self.define_holiday() \
            .with_names({
                "fi": "Juhannuspäivä",
                "sv": "Midsommardagen",
            }) \
            .on(first("saturday").after(month=6, day=19)) \
            .with_flags("NRV")

        """
        Saturday between 31 October and 6 November: All Saints' Day
        """
        self.define_holiday() \
            .with_names({
                "fi": "Pyhäinpäivä",
                "sv": "Alla helgons dag",
            }) \
            .on(first("saturday").after(month=10, day=30)) \
            .with_flags("NRV")



from holidata.holiday import Region
from holidata.utils import day


class BY(Region):
    """
    https://www.gesetze-bayern.de/Content/Document/BayFTG-9
    """
    def __init__(self, country):
        super().__init__("BY", country)

        self.define_holiday() \
            .with_name("Heilige drei Könige") \
            .on(month=1, day=6) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Mariä Himmelfahrt") \
            .on(month=8, day=15) \
            .with_flags("RF") \
            .annotated_with("In Gemeinden mit überwiegend katholischer Bevölkerung")

        self.define_holiday() \
            .with_name("Allerheiligen") \
            .on(month=11, day=1) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Fronleichnam") \
            .on(day(60).after(country.easter())) \
            .with_flags("RV")



from holidata.holiday import Region
from holidata.utils import day


class CT(Region):
    def __init__(self, country):
        super().__init__("CT", country)

        self.define_holiday() \
            .with_name("San Juan") \
            .in_years([2011, 2013, 2014, 2015, 2016, 2017, 2019, 2020, 2021, 2022, 2023, 2024]) \
            .on(month=6, day=24) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Fiesta Nacional de Cataluña") \
            .except_for([2011, 2016, 2022]) \
            .on(month=9, day=11) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes de Pascua Granada") \
            .in_years([2011, 2016, 2022]) \
            .on(day(50).after(country.easter())) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes de Pascua") \
            .except_for([2018]) \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("San Esteban") \
            .except_for([2018, 2021]) \
            .on(month=12, day=26) \
            .with_flags("RF")



from dateutil.easter import EASTER_WESTERN

from holidata.holiday import Country
from holidata.utils import SmartDayArrow

"""
sources:
law on national holidays and general holidays: https://www.mevzuat.gov.tr/mevzuat?MevzuatNo=2429&MevzuatTur=1&MevzuatTertip=5 
dates for holidays 'Ramazan Bayramı' and 'Kurban Bayramı': https://vakithesaplama.diyanet.gov.tr/dini_gunler.php
"""


class TR(Country):
    id = "TR"
    languages = ["tr"]
    default_lang = "tr"
    easter_type = EASTER_WESTERN

    def __init__(self):
        super().__init__()

        self.define_holiday() \
            .with_name("Yılbaşı") \
            .on(month=1, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Ulusal Egemenlik ve Çocuk Bayramı") \
            .on(month=4, day=23) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Emek ve Dayanışma Günü") \
            .on(month=5, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Atatürk'ü Anma, Gençlik ve Spor Bayramı") \
            .on(month=5, day=19) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Zafer Bayramı") \
            .on(month=8, day=30) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Cumhuriyet Bayramı") \
            .on(month=10, day=29) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Demokrasi ve Milli Birlik Günü") \
            .on(month=7, day=15) \
            .since(2017) \
            .with_flags("NF")

        """
        Ramazan Bayramı 1.-3. Gün
        """
        for i in [1, 2, 3]:
            self.define_holiday() \
                .with_name(f"Ramazan Bayramı ({i}. Gün)") \
                .on(TR.ramazan_bayrami_day(i)) \
                .with_flags("NRV")

        """
        Kurban Bayramı 1.-4. Gün
        """
        for i in [1, 2, 3, 4]:
            self.define_holiday() \
                .with_name(f"Kurban Bayramı ({i}. Gün)") \
                .on(TR.kurban_bayrami_day(i)) \
                .with_flags("NRV")

    @staticmethod
    def ramazan_bayrami_day(ordinal):
        def ramazan_bayrami_reference_shifted(year):
            reference = TR.__ramazan_bayrami_reference(year)
            return reference.shift(days=ordinal) if reference is not None else None

        return ramazan_bayrami_reference_shifted

    @staticmethod
    def kurban_bayrami_day(ordinal):
        def kurban_bayrami_reference_shifted(year):
            reference = TR.__kurban_bayrami_reference(year)
            return reference.shift(days=ordinal) if reference is not None else None

        return kurban_bayrami_reference_shifted

    @staticmethod
    def __ramazan_bayrami_reference(year):
        ramazan_bayrami_reference = {
            2011: SmartDayArrow(2011, 8, 29),
            2012: SmartDayArrow(2012, 8, 18),
            2013: SmartDayArrow(2013, 8,  7),
            2014: SmartDayArrow(2014, 7, 27),
            2015: SmartDayArrow(2015, 7, 16),
            2016: SmartDayArrow(2016, 7,  4),
            2017: SmartDayArrow(2017, 6, 24),
            2018: SmartDayArrow(2018, 6, 14),
            2019: SmartDayArrow(2019, 6,  4),
            2020: SmartDayArrow(2020, 5, 23),
            2021: SmartDayArrow(2021, 5, 12),
            2022: SmartDayArrow(2022, 5,  1),
            2023: SmartDayArrow(2023, 4, 20),
            2024: SmartDayArrow(2024, 4,  9),
            2025: SmartDayArrow(2025, 3, 29),
            2026: SmartDayArrow(2026, 3, 19),
            2027: SmartDayArrow(2027, 3,  8),
        }

        return ramazan_bayrami_reference.get(year)

    @staticmethod
    def __kurban_bayrami_reference(year):
        kurban_bayrami_reference = {
            2011: SmartDayArrow(2011, 11,  5),
            2012: SmartDayArrow(2012, 10, 24),
            2013: SmartDayArrow(2013, 10, 14),
            2014: SmartDayArrow(2014, 10,  3),
            2015: SmartDayArrow(2015,  9, 23),
            2016: SmartDayArrow(2016,  9, 11),
            2017: SmartDayArrow(2017,  8, 31),
            2018: SmartDayArrow(2018,  8, 20),
            2019: SmartDayArrow(2019,  8, 10),
            2020: SmartDayArrow(2020,  7, 30),
            2021: SmartDayArrow(2021,  7, 19),
            2022: SmartDayArrow(2022,  7,  8),
            2023: SmartDayArrow(2023,  6, 27),
            2024: SmartDayArrow(2024,  6, 15),
            2025: SmartDayArrow(2025,  6,  5),
            2026: SmartDayArrow(2026,  5, 26),
            2027: SmartDayArrow(2027,  5, 15),
        }

        return kurban_bayrami_reference.get(year)



from dateutil.easter import EASTER_WESTERN

from holidata.holiday import Country
from holidata.utils import day


class SK(Country):
    id = "SK"
    languages = ["sk"]
    default_lang = "sk"
    easter_type = EASTER_WESTERN

    def __init__(self):
        super().__init__()

        self.define_holiday() \
            .with_name("Deň vzniku Slovenskej republiky") \
            .on(month=1, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Zjavenie Pána / Traja králi") \
            .on(month=1, day=6) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Sviatok práce") \
            .on(month=5, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Deň víťazstva nad fašizmom") \
            .on(month=5, day=8) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Sviatok svätého Cyrila a Metoda") \
            .on(month=7, day=5) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Výročie SNP") \
            .on(month=8, day=29) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Deň Ústavy Slovenskej republiky") \
            .on(month=9, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Sedembolestná Panna Mária") \
            .on(month=9, day=15) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Sviatok všetkých svätých") \
            .on(month=11, day=1) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Deň boja za slobodu a demokraciu") \
            .on(month=11, day=17) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Štedrý deň") \
            .on(month=12, day=24) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Prvý sviatok vianočný") \
            .on(month=12, day=25) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Druhý sviatok vianočný") \
            .on(month=12, day=26) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Veľký piatok") \
            .on(day(2).before(self.easter())) \
            .with_flags("NRV")

        self.define_holiday() \
            .with_name("Veľkonočný pondelok") \
            .on(day(1).after(self.easter())) \
            .with_flags("NRV")



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, day


class RI(Region):
    def __init__(self, country):
        super().__init__("RI", country)

        self.define_holiday() \
            .with_name("Día de La Rioja") \
            .on(RI.dia_de_la_rioja) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes siguiente al Día de La Rioja") \
            .in_years([2024]) \
            .on(month=6, day=10) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("San José") \
            .in_years([2012]) \
            .on(month=3, day=19) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Jueves Santo") \
            .in_years([2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2024]) \
            .on(day(3).before(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Lunes de Pascua") \
            .except_for([2012, 2018]) \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Santiago Apóstol") \
            .in_years([2011, 2016]) \
            .on(month=7, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Lunes siguiente al Día de la Constitución Española") \
            .in_years([2015, 2020]) \
            .on(month=12, day=7) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes siguiente a La Inmaculada Concepción") \
            .in_years([2013, 2019]) \
            .on(month=12, day=9) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Lunes siguiente a la Natividad del Señor") \
            .in_years([2022]) \
            .on(month=12, day=26) \
            .with_flags("RF")

    @staticmethod
    def dia_de_la_rioja(year):
        if year in [2013, 2019]:
            return SmartDayArrow(year, 6, 10)
        else:
            return SmartDayArrow(year, 6, 9)
